package com.wechi.adweb.bridge.google.analytics;

import com.google.analytics.data.v1beta.BetaAnalyticsDataClient;
import com.google.analytics.data.v1beta.BetaAnalyticsDataSettings;
import com.google.analytics.data.v1beta.Dimension;
import com.google.analytics.data.v1beta.Metric;
import com.google.api.gax.core.FixedCredentialsProvider;
import com.google.auth.oauth2.GoogleCredentials;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.analytics.dto.GAReportRequestDTO;

import jakarta.annotation.PostConstruct;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.util.List;

@Slf4j
@Data
public class GADataService {

    private BetaAnalyticsDataSettings dataSettings;

    private Gson gson = new Gson();

    @PostConstruct
    private void init() throws IOException {
        GoogleCredentials credentials =
                GoogleCredentials.fromStream(
                        this.getClass()
                                .getClassLoader()
                                .getResourceAsStream("google/service-account-key.json"));

        this.dataSettings =
                BetaAnalyticsDataSettings.newBuilder()
                        .setCredentialsProvider(FixedCredentialsProvider.create(credentials))
                        .build();
    }

    List<Object> runReport(GAReportRequestDTO reportRequest) throws DataException {
        try (BetaAnalyticsDataClient analyticsDataClient =
                BetaAnalyticsDataClient.create(dataSettings)) {
            // 1. Dimensions.
            List<Dimension> dimensions =
                    Splitter.on(GAReportRequestDTO.DEFAULT_SPLITTER)
                            .splitToStream(Strings.nullToEmpty(reportRequest.getDimensions()))
                            .map(dimension -> Dimension.newBuilder().setName(dimension).build())
                            .toList();

            // 2. Metrics.
            List<Metric> metrics =
                    Splitter.on(GAReportRequestDTO.DEFAULT_SPLITTER)
                            .splitToStream(Strings.nullToEmpty(reportRequest.getMetrics()))
                            .map(metric -> Metric.newBuilder().setName(metric).build())
                            .toList();

        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }

        return null;
    }
}
